/*
 * ******************************************************************************************
 *
 * File		: main.c
 * Author	: GOWIN
 * Function	: APB Timer demo
 *
 * ******************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "apb_timer_driver.h"
#include "apb_timer_dev.h"
#include "uart.h"
#include <stdio.h>


// Definitions ------------------------------------------------------------------------------

volatile unsigned int counter;


// Initializes APB Timer
void apb_timer_init(void)
{
	printf("Initializes APB Timer...\r\n");

	DEV_APB_TIMER->INTCLEAR = 0;
	DEV_APB_TIMER->RELOAD   = 25000000;					// Set reload value
	DEV_APB_TIMER->VALUE    = DEV_APB_TIMER->RELOAD; 	// Set the current value
	DEV_APB_TIMER->CTRL     = 0;  						// Disable interrupt
}

// Setup APB Timer
void apb_timer_setup(void)
{
	printf("Setup APB Timer...\r\n");

	// Set the interrupt level 1 for the APB Timer
	HAL_INTERRUPT_SET_LEVEL(IRQ_APB_TIMER_SOURCE, 1);

	/* Enable PLIC interrupt APB Timer source */
	HAL_INTERRUPT_ENABLE(IRQ_APB_TIMER_SOURCE);

	// Enable APB Timer interrupt
	apb_timer_enable_irq(DEV_APB_TIMER);
}

// APB Timer interrupt handler
void apb_timer_irq_handler(void)
{
	if(apb_timer_get_irq_status(DEV_APB_TIMER) != RESET)
	{
		counter++;
		apb_timer_clear_irq(DEV_APB_TIMER);
	}
}

// Application entry function
int main(void)
{
	int cnt_num = 0;

	// Initializes UART
	uart_init(38400);		// Baud rate is 38400

	printf("It's an APB Timer demo.\r\n\r\n");

	// Initializes APB Timer
	apb_timer_init();

	counter = 0;

	// Disable Machine External interrupts
  	HAL_MEIP_DISABLE();

  	// Setup APB Timer
  	apb_timer_setup();

  	// Enable Machine External interrupts
  	HAL_MEIP_ENABLE();

	// Enable interrupts in general.
	HAL_MIE_ENABLE();

	// Start APB Timer
	printf("Start APB Timer...\r\n\r\n");
	apb_timer_start(DEV_APB_TIMER);

	while(1)
	{
		if(counter == 2)
		{
			counter = 0;
			printf("Counter number: %d\r\n", cnt_num);
			cnt_num++;
		}

		if(cnt_num == 60)
		{
			cnt_num = 0;
		}
	}
}
